#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import can
import logging

logger = logging.getLogger(__name__)


class PythonCanDriver:
    """
    This class act as a *CAN* bus interface in order to communicate with the *Xcom-CAN*, it is **NOT**
    intended to access it directly.

    Attributes
    ----------
    can_bus : can.interface.Bus
        low-level device specific CAN interface
    """

    def __init__(self, bitrate, bustype='kvaser'):
        """
        Initialize PythonCanDriver.can_bus with the specified bitrate for a *Kvaser* interface if running on Windows,
        otherwise you must specify your bustype interface such as *SocketCAN* (not tested yet)

        Parameters
        ----------
        bitrate
            CAN bus speed
        """
        if os.name == 'nt':
            self.can_bus = can.interface.Bus(bustype=bustype, channel=0, bitrate=bitrate)
        else:
            self.can_bus = can.interface.Bus(bitrate=bitrate)

    def __del__(self):
        self.can_bus.shutdown()

    def receive(self, timeout=100):
        """
        Read a message on the CAN bus, waiting block

        Parameters
        ----------
        timeout : float
            milli seconds to wait for a message (default 100ms)

        Returns
        -------
        tuple
            identifier, data, dlc, flag, timestamp
        """
        message = self.can_bus.recv(timeout=timeout / 1000)
        if message is None:
            return None, None, None, None, None
        else:
            logger.debug("<- rx : id 0x" + str(format(message.arbitration_id, '08x')) + " | data 0x" + str(
                bytes(message.data).hex()) + " | DLC " + str(message.dlc))
            return message.arbitration_id, message.data, message.dlc, None, message.timestamp

    def send(self, identifier, data, is_extended_id=False):
        """
        Send a message on the CAN bus

        Parameters
        ----------
        identifier : int
            The frame identifier used for arbritration on the bus

        data : bytearray
            The data parameter of a CAN message, length from 0 to 8 bytes

        is_extended_id : bool
            Set usage of CAN2.0A (Standard, 11 bits identifier) of CAN2.0B (extended, 29 bits identifier)
        """
        message = can.Message(arbitration_id=identifier, data=data, is_extended_id=is_extended_id)
        logger.debug("-> tx : id 0x" + str(format(message.arbitration_id, '08x')) + " | data 0x" + str(
            bytes(message.data).hex()) + " | DLC " + str(message.dlc))
        self.can_bus.send(message)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


from threading import Thread
from struct import unpack, calcsize


class Timeout(Exception):
    """
    This class rises timeout exceptions
    """
    pass


class Service(object):
    """
    This base class represents a service
    """

    @classmethod
    def from_bytes(cls, buffer):
        """
        Unpack Service data buffer
        """
        format_size = calcsize(cls.PACK_FORMAT)
        return cls(*unpack(cls.PACK_FORMAT, buffer[:format_size]), buffer[format_size:])

    def __repr__(self):
        return "{}{}".format(type(self).__name__, vars(self))

    def handle(self, source_address, destination_address, exeption=None):
        """
        Method intended to be overriden by childs
        """
        pass


class CanNode(Thread):
    """
    This base class let a can node run in a single thread

    Attributes
    ----------
    driver: PythonCanDriver
        access low-level CAN bus
    address: int
        CAN node address
    services: list
        a list of services
    isRunning: boolean
        state of the CAN node
    """

    def __init__(self, driver, address):
        """
        Initialize Thread

        Parameters
        ----------
        driver : PythonCanDriver
            Relying can interface

        address : int
            Node CAN address
        """
        Thread.__init__(self)
        self.driver = driver
        self.address = address
        self.services = []
        self.isRunning = True

    def stop(self):
        """
        Set CanNode.isRunning boolean class variable to False
        """
        self.isRunning = False

    def add_service(self, handler):
        """
        Add service handler to the CAN node
        """
        self.services.append(handler)

    def run(self):
        """
        Override method from Thread to permanently read data on the CAN bus if CanNode.isRunning is True
        """
        while self.isRunning:
            identifier, data, dlc, flag, time = self.driver.receive()
            if identifier is None:
                continue
            self.handle_rx_frame(identifier, data, dlc, flag, time)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys, os
import setuptools

current_directory = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(current_directory, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

name = "stucancommon"
version = "0.9"
release = "0.9.2"

setuptools.setup(
    name=name,
    version=release,
    author="Studer Innotec SA",
    author_email="develop@studer-innotec.com",
    maintainer_email="develop@studer-innotec.com",
    description="Package that let access a CAN bus",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://github.com/studer-innotec/stucancommon",
    project_urls={
        "Documentation": "https://stucancommon.readthedocs.io/en/latest/index.html",
        "Issues tracker": "https://github.com/studer-innotec/stucancommon/issues",
        "Source Code": "https://github.com/studer-innotec/stucancommon",
    },
    packages=setuptools.find_packages(),
    include_package_data=True,
    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        'Topic :: Communications',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    python_requires='>=3.6.8',
    install_requires=['python-can>=3.3.2'],
    # these are optional and override conf.py settings
    command_options={
        'build_sphinx': {
            'project': ('setup.py', name),
            'version': ('setup.py', version),
            'release': ('setup.py', release),
            'source_dir': ('setup.py', 'docs/source'),
            'build_dir': ('setup.py', 'docs/build'),
            'all_files': ('setup.py', 1)},
    },
)
